{***********************************************************************}
{                                                                       }
{   Module Name: Ws2tcpip6.pas                                          }
{                                                                       }
{   Version:  0.1, April 7th, 2000                                      }
{                                                                       }
{   Abstract: WinSock2 TCP/IP6 specifics definions                      }
{             for MSR IPv6. It is a (partial) translation to Delphi of  }
{             Microsoft Research's Wstcpip.h and ws2ip6.h               }
{                                                                       }
{   Authors:  David Fernandez and Javier Lucendo                        }
{             Telematics Department                                     }
{             Technical University of Madrid                            }
{             (c) 2000 by DIT-UPM                                       }
{             email: david@dit.upm.es                                   }
{             www:   www.dit.upm.es/bti                                 }
{                                                                       }
{***********************************************************************}

{$MINENUMSIZE 4} { Force 4 bytes enumeration size}

unit WS2TCPIP6;

interface

uses Windows, WinSock2, Sysutils;

const
  IPPROTO_IPV6 = 41;

type
  u_char = Char;
  u_short = Word;
  u_int = Integer;
  u_long = Longint;
  pu_long = ^u_long;
  pu_short = ^u_short;

{ The new type to be used in all
  instances which refer to sockets. }
  TSocket = u_int;

type

//
// The IP APIs return status codes of this type. This is implementation-specific for Microsoft
//
  IP_STATUS = longint;

  SunB6 = packed record
    s_b1,  s_b2,  s_b3,  s_b4,
    s_b5,  s_b6,  s_b7,  s_b8,
    s_b9,  s_b10, s_b11, s_b12,
    s_b13, s_b14, s_b15, s_b16: u_char;
  end;

  SunW6 = packed record
    s_w1, s_w2, s_w3, s_w4,
    s_w5, s_w6, s_w7, s_w8: u_short;
  end;

  SunDW6 = packed record
    s_dw1, s_dw2, s_dw3, s_dw4: longint;
  end;

  S6_Bytes   = SunB6;
  S6_Words   = SunW6;
  S6_DWords  = SunDW6;
  S6_Addr    = SunB6;  // The only member defined in the draft.


  PInAddr6 = ^TInAddr6;
  TInAddr6 = packed record
    case integer of
      0: (S_un_b:  SunB6);
      1: (S_un_w:  SunW6);
      2: (S_un_dw: SunDW6);
  end;


  PSockAddrIn6 = ^TSockAddrIn6;
  TSockAddrIn6 = packed record
		sin6_family:   u_short;     // AF_INET6
		sin6_port:     u_short;     // Transport level port number
		sin6_flowinfo: u_long;	    // IPv6 flow information
		sin6_addr:     TInAddr6;    // IPv6 address
		sin6_scope_id: u_long;      // Scope Id: IF number for link-local
                                    //           SITE id for site-local
  end;


  TIn6_Addr  = TInAddr6;
  PIn6_Addr  = PInAddr6 ;

  TSockAddr_In6  = TSockAddrIn6;
  PSockAddr_In6  = PSockAddrIn6;


  TIPv6_mreq = record
                  ipv6mr_multiaddr: TInAddr6; // IPv6 multicast address.
                  ipv6mr_interface: u_long;  // Interface index.
                  padding: u_long;
               end;

  // These functions were "inline" in the original header
  function IN6_IS_ADDR_UNSPECIFIED(const a: PInAddr6): boolean;
  function IN6_IS_ADDR_LOOPBACK(const a: PInAddr6): boolean;
  function IN6_IS_ADDR_LINKLOCAL(const a: PInAddr6): boolean;
  function IN6_IS_ADDR_SITELOCAL(const a: PInAddr6): boolean;
  function IN6_IS_ADDR_MULTICAST(const a: PInAddr6): boolean;
  function IN6_ADDR_EQUAL(const a: PInAddr6; const b: PInAddr6):boolean;

  procedure SET_IN6_IF_ADDR_ANY (const a: PInAddr6);
  procedure SET_LOOPBACK_ADDR6 (const a: PInAddr6);


const
  //
  // Flag values for getipnodebyname().
  //
  AI_V4MAPPED   = 1;
  AI_ALL        = 2;
  AI_ADDRCONFIG = 4;
  AI_DEFAULT    = AI_V4MAPPED + AI_ADDRCONFIG;

  INET_ADDRSTRLEN  = 16;
  INET6_ADDRSTRLEN = 46;

//
// Socket options at the IPPROTO_IPV6 level.
//
const

  IPV6_MULTICAST_IF      = 9;  // set/get IP multicast i/f
  IPV6_MULTICAST_HOPS    = 10; // set/get IP multicast ttl
  IPV6_MULTICAST_LOOP    = 11; // set/get IP multicast loopback
  IPV6_ADD_MEMBERSHIP    = 12; // add an IP group membership
  IPV6_DROP_MEMBERSHIP   = 13; // drop an IP group membership

//const
//   IN6ADDR_ANY      = $00000000000000000000000000000000;
//   IN6ADDR_LOOPBACK = $00000000000000000000000000000001;
var
  in6addr_any, in6addr_loopback : TInAddr6;

//
// getaddrinfo types and consts
//
const
  //
  // Error codes from getaddrinfo().
  //
  EAI_ADDRFAMILY  = 1;   // Address family for nodename not supported.
  EAI_AGAIN       = 2;   // Temporary failure in name resolution.
  EAI_BADFLAGS    = 3;   // Invalid value for ai_flags.
  EAI_FAIL        = 4;   // Non-recoverable failure in name resolution.
  EAI_FAMILY      = 5;   // Address family ai_family not supported.
  EAI_MEMORY      = 6;   // Memory allocation failure.
  EAI_NODATA      = 7;   // No address associated with nodename.
  EAI_NONAME      = 8;   // Nodename nor servname provided, or not known.
  EAI_SERVICE     = 9;   // Servname not supported for ai_socktype.
  EAI_SOCKTYPE    = 10;  // Socket type ai_socktype not supported.
  EAI_SYSTEM      = 11;  // System error returned in errno.

type
  //
  // Structure used in getaddrinfo() call.
  //
  TAddrInfo = record
                ai_flags: integer;    // AI_PASSIVE, AI_CANONNAME, AI_NUMERICHOST.
                ai_family: integer;   // PF_xxx.
                ai_socktype: integer; // SOCK_xxx.
                ai_protocol: integer; // 0 or IPPROTO_xxx for IPv4 and IPv6.
                ai_addrlen: u_int;    // Length of ai_addr.
                ai_canonname: PChar;  // Canonical name for nodename.
                ai_addr: PSockAddr;   // Binary address.
                ai_next: Pointer;     // Next structure in linked list.
              end;
  PAddrInfo = ^TAddrInfo;

const
  //
  // Flags used in "hints" argument to getaddrinfo().
  //
  AI_PASSIVE     = $1;  // Socket address will be used in bind() call.
  AI_CANONNAME   = $2;  // Return canonical name in first ai_canonname.
  AI_NUMERICHOST = $4;  // Nodename must be a numeric address string.


//
// WSHIP6.DLL Function types
//
type
  TGetAddrInfo     = function (NodeName: PChar; ServName: PChar; Hints: PAddrInfo;
                           var Addrinfo: PAddrInfo): integer; stdcall;
  TFreeAddrInfo    = procedure (ai: PAddrInfo); stdcall;
  TGetErrorString  = function (Error: IP_STATUS): PChar; stdcall;
  TGetIPNodeByName = function (const name: PChar; af: integer; flags: integer): PHostEnt; stdcall;
  Tinet6_ntoa      = function (const Address: PInAddr6): PChar; stdcall;
  Tinet6_addr      = function (const s: PChar; const Address: PInAddr6): integer; stdcall;
  Tinet_ntop       = function (AddressFamily: integer; Address: PChar;
                               OutputString: PChar; OutputBufferSize: integer): PChar; stdcall;
  Tinet_pton       = function (AddressFamily: integer; InputString: PChar; Address: Pointer): integer; stdcall;
  TFreeHostEnt     = procedure (ptr: PHostEnt); stdcall;


var
  IPv6Supported: boolean;
  WSHIP6Lib : HMODULE;

  //
  // WSHIP6.DLL Function variables
  //

  // int getaddrinfo(const char *nodename, const char *servname,
  //            const struct addrinfo *hints, struct addrinfo **res);
  // function getaddrinfo (
  //           NodeName: PChar;
  //           ServName: PChar;
  //           Hints: PAddrInfo;
  //           var Addrinfo: PAddrInfo
  //           ): integer; stdcall;
  GetAddrInfo     : TGetAddrInfo;

  //procedure freeaddrinfo(ai: PAddrInfo); stdcall;
  //
  FreeAddrInfo    : TFreeAddrInfo;

  GetErrorString  : TGetErrorString;

  GetIPNodeByName : TGetIPNodeByName;
  //* inet6_ntoa - Converts a binary IPv6 address into a string.
  //               This function is NOT defined in RFC 2553.
  //               Use inet_ntop (below) if you want to be portable.
  //               Returns a pointer to the output string.
  // function inet6_ntoa(
  //         const Address: PInAddr6  // Address (binary) to convert.
  //         ): PChar; stdcall;
  inet6_ntoa      : Tinet6_ntoa;

  //* inet6_addr - Converts a string into a binary IPv6 address.
  //               This function is NOT defined in RFC 2553.
  //               Use inet_pton (below) if you want to be portable.
  //               Returns TRUE if successful, FALSE otherwise.
  //  function inet6_addr(
  //         const s: PChar;         // IPv6 address (in "colon" representation).
  //         const Address: PInAddr6 // Where to return binary representation.
  //         ): integer; stdcall;
  inet6_addr      : Tinet6_addr;

  //* inet_ntop - Converts a binary address into a string.
  //              As specified in RFC 2553, Section 6.6.
  //              Returns a pointer to the output string if successful,
  //              NULL otherwise.
  // function inet_ntop (
  //         AddressFamily: integer;    // Address family to which the address belongs.
  //         Address: PChar;            // Address (binary) to convert.
  //         OutputString: PChar;       // Where to return the output string.
  //         OutputBufferSize: integer  // Size of above buffer.
  //         ): PChar; stdcall;
  inet_ntop       : Tinet_ntop;

  //* inet_pton - Converts an address from a string to binary.
  //              As specified in RFC 2553, Section 6.6.
  //              Returns 1 upon success, 0 upon invalid string,
  //              and -1 for unknown address family.
  // function inet_pton (
  //         AddressFamily: integer;    // Address family to which the address belongs.
  //         InputString: PChar;        // Address (numeric string) to convert.
  //         Address: Pointer           // Where to return the binary address.
  //         ): integer; stdcall;}
  inet_pton       : Tinet_pton;

  //  procedure freehostent(ptr: PHostEnt); stdcall;
  //
  FreeHostEnt     : TFreeHostEnt;


////////////////////////////////////////////////////////////
//             I M P L E M E N T A T I O N                //
////////////////////////////////////////////////////////////

implementation


  function IN6_IS_ADDR_UNSPECIFIED(const a: PInAddr6): boolean;
  begin
    Result := ((a^.s_un_dw.s_dw1 = 0) and (a^.s_un_dw.s_dw2 = 0) and
               (a^.s_un_dw.s_dw3 = 0) and (a^.s_un_dw.s_dw4 = 0));
  end;

  function IN6_IS_ADDR_LOOPBACK(const a: PInAddr6): boolean;
  begin
    Result := (
      (a^.s_un_b.s_b1 = char(0))  and (a^.s_un_b.s_b2 = char(0))  and
      (a^.s_un_b.s_b3 = char(0))  and (a^.s_un_b.s_b4 = char(0))  and
      (a^.s_un_b.s_b5 = char(0))  and (a^.s_un_b.s_b6 = char(0))  and
      (a^.s_un_b.s_b7 = char(0))  and (a^.s_un_b.s_b8 = char(0))  and
      (a^.s_un_b.s_b9 = char(0))  and (a^.s_un_b.s_b10 = char(0)) and
      (a^.s_un_b.s_b11 = char(0)) and (a^.s_un_b.s_b12 = char(0)) and
      (a^.s_un_b.s_b13 = char(0)) and (a^.s_un_b.s_b14 = char(0)) and
      (a^.s_un_b.s_b15 = char(0)) and (a^.s_un_b.s_b16 = char(1)) );
  end;

  function IN6_IS_ADDR_LINKLOCAL(const a: PInAddr6): boolean;
  begin
    Result := ((a^.s_un_b.s_b1 = u_char($FE)) and (a^.s_un_b.s_b2 = u_char($80)));
  end;

  function IN6_IS_ADDR_SITELOCAL(const a: PInAddr6): boolean;
  begin
    Result := ((a^.s_un_b.s_b1 = u_char($FE)) and (a^.s_un_b.s_b2 = u_char($C0)));
  end;

  function IN6_IS_ADDR_MULTICAST(const a: PInAddr6): boolean;
  begin
    Result := (a^.s_un_b.s_b1 = char($FF));
  end;

  function IN6_ADDR_EQUAL(const a: PInAddr6; const b: PInAddr6): boolean;
  begin
    Result := (CompareMem( a, b, sizeof(TInAddr6) ) );
  end;

  procedure SET_IN6_IF_ADDR_ANY (const a: PInAddr6);
  begin
    FillMemory (a, sizeof(TInAddr6), 0);
    {a^.s_un_dw.s_dw1 := 0;
    a^.s_un_dw.s_dw2 := 0;
    a^.s_un_dw.s_dw3 := 0;
    a^.s_un_dw.s_dw4 := 0;}
  end;

  procedure SET_LOOPBACK_ADDR6 (const a: PInAddr6);
  begin
    FillMemory (a, sizeof(TInAddr6), 0);
    {a^.s_un_b.s_b1 := char(0);  a^.s_un_b.s_b2 := char(0);
    a^.s_un_b.s_b3 := char(0);  a^.s_un_b.s_b4 := char(0);
    a^.s_un_b.s_b5 := char(0);  a^.s_un_b.s_b6 := char(0);
    a^.s_un_b.s_b7 := char(0);  a^.s_un_b.s_b8 := char(0);
    a^.s_un_b.s_b9 := char(0);  a^.s_un_b.s_b10 := char(0);
    a^.s_un_b.s_b11 := char(0); a^.s_un_b.s_b12 := char(0);
    a^.s_un_b.s_b13 := char(0); a^.s_un_b.s_b14 := char(0);
    a^.s_un_b.s_b15 := char(0);}
    a^.s_un_b.s_b16 := char(1);
  end;

  // functions exported by the dll wship6.dll. There are some more (up to 20)
  // that are not included here.
  //  function GetErrorString(Error: IP_STATUS): PChar; external 'wship6.dll';
  //  function getipnodebyname(const name: PChar; af: integer; flags: integer): PHostEnt;  external 'wship6.dll';
  //  procedure freehostent(ptr: PHostEnt); external 'wship6.dll';
  //  function inet6_ntoa(const Address: PInAddr6): PChar;  external 'wship6.dll';
  //  function inet6_addr(const s: PChar; const Address: PInAddr6): integer;  external 'wship6.dll';
  //  function inet_ntop (AddressFamily: integer; Address: PChar; OutputString: PChar;
  //    OutputBufferSize: integer): PChar; external 'wship6.dll';
  //  function inet_pton (AddressFamily: integer; InputString: PChar;
  //    Address: Pointer): integer; stdcall; external 'wship6.dll';
  //  function getaddrinfo (NodeName: PChar; ServName: PChar; Hints: PAddrInfo;
  //    var Addrinfo: PAddrInfo): integer; stdcall; external 'wship6.dll';
  //  procedure freeaddrinfo(ai: PAddrInfo); stdcall; external 'wship6.dll';

begin
  SET_IN6_IF_ADDR_ANY (@in6addr_any);
  SET_LOOPBACK_ADDR6  (@in6addr_loopback);

  WSHIP6Lib := LoadLibrary ('wship6.dll');
  IPv6Supported :=  (WSHIP6Lib <> 0);

  if IPv6Supported then begin
    GetAddrInfo     := GetProcAddress (WSHIP6Lib, 'getaddrinfo');
    FreeAddrInfo    := GetProcAddress (WSHIP6Lib, 'freeaddrinfo');
    GetErrorString  := GetProcAddress (WSHIP6Lib, 'geterrorstring');
    GetIPNodeByName := GetProcAddress (WSHIP6Lib, 'getipnodebyname');
    inet6_ntoa      := GetProcAddress (WSHIP6Lib, 'inet6_ntoa');
    inet6_addr      := GetProcAddress (WSHIP6Lib, 'inet6_addr');
    inet_ntop       := GetProcAddress (WSHIP6Lib, 'inet_ntop');
    inet_pton       := GetProcAddress (WSHIP6Lib, 'inet_pton');
    FreeHostEnt     := GetProcAddress (WSHIP6Lib, 'freehostent');;
  end;
end.
